/*
 * Getrap
 * Copyright 2008(C) Grzegorz 'milyges' Glinski
 *
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
*/
#ifndef __CONFIG_H
#define __CONFIG_H

#include <QString>
#include <QStringList>
#include <QList>
#include <QFile>

struct configItem
{
 /** Nazwa wpisu */
 QString name;
 /** Wartość wpisu */
 QString value;
};

/**
 * Klasa dostarczająca pełną obsługę konfiguracji programu jak wczytywanie, edytowanie czy zapisywanie pliku konfiguracyjnego
*/
class Config
{
 private:
   /** Tablica konfiguracji. */
   static QList<struct configItem> mData;
   /**
    * Pobierz ścieszkę do pliku konfiguracyjnego
    * @return ścieszka do pliku konfiguracyjnego
   */
   static QString getPath(void);
 public:
   /**
    * Wczytaj plik konfiguracyjny do pamięci
   */
   static bool load();
   /**
    * Zapisz plik konfiguracyjny na dysk
   */
   static bool save();
   /**
    * Funkcja służąca do pobierania wartości z tablicy konfiguracji
    * @param name nazwa wpisu w tablicy
    * @return wartość wpisu lub "" gdy wpis nie istnieje
   */
   static QString getValue(QString name);
   /**
    * Funkcja zmienia wartość wpisu w tablicy konfiguracji lub tworzy nowy wpis gdy nie istnieje
    * @param name nazwa wpisu
    * @param value wartość wpisu
   */
   static void setValue(QString name, QString value);
};

#endif
